//GUI for controlling USB Tiny LED
//Copyright Brian Starkey

#include <glib.h>
#include <gdk/gdk.h>
#include <gtk/gtk.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <usb.h>        /* this is libusb */
#include "opendevice.h" /* common code moved to separate module */
#include "../firmware/requests.h"   /* custom request numbers */
#include "../firmware/usbconfig.h"  /* device's VID/PID and names */


/* Initialisations */

GtkWidget *coloursel = NULL;	//Global reference to colour selection widget
GtkWidget *slider = NULL;		//Global reference to slider widget
GdkColor color;					//Global color oobject
usb_dev_handle *handle = NULL;	//Global USB Device handle


/* Function Prototypes */

void send_settings(GdkColor sColour, unsigned char speed);
void start_usb();
static void color_changed_cb( 	GtkWidget *widget, 
								GtkColorSelection *colorsel);
static void colour_stuff() ;
static gboolean destroy_window( GtkWidget *widget,
                                GdkEvent  *event,
                                gpointer   client_data );

//==================================================================

gboolean  expose_event_callback (GtkWidget *widget, GdkEventExpose *event, gpointer data)
{
	GdkColor tcolor;
	gtk_color_selection_get_current_color (GTK_COLOR_SELECTION(coloursel), &tcolor);	
    return TRUE;
}

//==================================================================





/* void start_usb() 
 * 		Opens the tinyLED USB device and assigns 'handle'
 */ 

void start_usb() {
	int vid, pid;
	const unsigned char rawVid[2] = {USB_CFG_VENDOR_ID};
	const unsigned char rawPid[2] = {USB_CFG_DEVICE_ID};
	char vendor[] = {USB_CFG_VENDOR_NAME, 0};
	char product[] = {USB_CFG_DEVICE_NAME, 0};

	usb_init();
    /* compute VID/PID from usbconfig.h so that there is a central source of information */
    vid = rawVid[1] * 256 + rawVid[0];
    pid = rawPid[1] * 256 + rawPid[0];
    /* The following function is in opendevice.c: */
    if(usbOpenDevice(&handle, vid, vendor, pid, product, NULL, NULL, NULL) != 0){
		fprintf(stderr, "Could not find USB device \"%s\" with vid=0x%x pid=0x%x\n", product, vid, pid);
		exit(1);
    }
}

/* void send_settings(GdkColor sColour, unsigned char speed)
 * 		Sends the necessary control message to USB Device 'handle' (global)
 * 		sColour is the colour information
 * 		speed is the fade speed (0 is instant, 255 is very slow)
 */

void send_settings(GdkColor sColour, unsigned char speed) 
{
	int cnt;
	char buffer[4];

	int rVal = sColour.red / 256;	// GdkColors use 16 bit colours
	int gVal = sColour.green / 256;	// The tinyLED only uses 8 bit values
	int bVal = sColour.blue / 256;	// for each component
	
	/* wValue and wIndex are 16 bit words, so can carry 2 bytes each....
	 * Perfect for the 4 bytes needed to control the tinyLED
	 */
	int wV = gVal * 256;			// wValue.bytes[1] = green
	int wI = speed * 256;			// wIndex.bytes[1] = speed

	wV = wV | rVal;					// wValue.bytes[0] = red
	wI = wI | bVal;					// wIndex.bytes[0] = blue
	
	cnt = usb_control_msg(handle, USB_TYPE_VENDOR | USB_RECIP_DEVICE | USB_ENDPOINT_OUT, CUSTOM_RQ_SET_COLOUR, wV, wI, buffer, 0, 5000);

}



/* Color changed handler */

static void color_changed_cb( GtkWidget         *widget,
                              GtkColorSelection *colorsel )
{
	GdkColor ncolor;
	unsigned char speed;

	gtk_color_selection_get_current_color (colorsel, &ncolor);
	speed = gtk_range_get_value(GTK_RANGE(slider));
	send_settings(ncolor, speed);
        
}

/* static void colour_stuff()
 * 		Create and initialise the GtkColorSelection widget
 * 		Set the initial colour to pure blue, no opacity control
 * 		Set the callback for every time the colour is changed
 */
static void colour_stuff() 
{
	GtkColorSelection *colourselp = NULL;

	coloursel = gtk_color_selection_new ();
	colourselp = GTK_COLOR_SELECTION (coloursel);
    gtk_color_selection_set_previous_color (colourselp, &color);
    gtk_color_selection_set_current_color (colourselp, &color);
    gtk_color_selection_set_has_palette (colourselp, FALSE);
	gtk_color_selection_set_has_opacity_control (colourselp, FALSE);
    g_signal_connect (G_OBJECT (colourselp), "color_changed", G_CALLBACK (color_changed_cb), (gpointer) colourselp);
    color.red = 0;
	color.blue = 65535;
	color.green = 0;
	gtk_color_selection_set_current_color (colourselp, &color);
  
}


/* Close down and exit handler */
static gboolean destroy_window( GtkWidget *widget,
                                GdkEvent  *event,
                                gpointer   client_data )
{
  gtk_main_quit ();
  return TRUE;
}



gint main( gint   argc,
           gchar *argv[] )
{
	GtkWidget *window, *box, *slabel;
	
	gtk_init (&argc, &argv);
	
	
	window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title (GTK_WINDOW (window), "Set LED Colour");
	gtk_window_set_policy (GTK_WINDOW (window), FALSE, FALSE, TRUE);
	gtk_window_set_position (GTK_WINDOW (window), GTK_WIN_POS_CENTER);
	g_signal_connect (	GTK_OBJECT (window), "delete_event",
						GTK_SIGNAL_FUNC (destroy_window), (gpointer) window);

	start_usb();


  GtkWidget *drawing_area = gtk_drawing_area_new ();
  gtk_widget_set_size_request (drawing_area, 100, 100);
  g_signal_connect (G_OBJECT (drawing_area), "expose_event",
                    G_CALLBACK (expose_event_callback), NULL);

	box = gtk_vbox_new ( FALSE, 5);
	slabel = gtk_label_new("Fade Time (low is fast):");
	slider = gtk_hscale_new_with_range(0,255, 1);
	GtkScale *sslider = GTK_SCALE(slider);
	GtkRange *rslider = GTK_RANGE(sslider);
	
	gtk_range_set_value(GTK_RANGE(rslider), 5);
	colour_stuff();
	gtk_box_pack_start( GTK_BOX (box), coloursel, FALSE, TRUE, 5);
	gtk_box_pack_start( GTK_BOX (box), slabel, FALSE, FALSE, 0);
	gtk_box_pack_start( GTK_BOX (box), slider, FALSE, TRUE, 0);
	gtk_container_add (GTK_CONTAINER (window), box);
	gtk_widget_show (slabel);
	gtk_widget_show (slider);
	gtk_widget_show (coloursel);
	gtk_widget_show (box);
	gtk_widget_show (window);

	gtk_main ();
	return 0;
}
